/*
 * InSyTrack - software for tracking program flow.
 *	-- example client with random calls.
 *
 * Copyright (C) 2012-2013 Bogdan Drozdowski, bogdandr (at) op.pl
 * License: GNU Lesser General Public License, v3+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foudation:
 *		Free Software Foundation
 *		51 Franklin Street, Fifth Floor
 *		Boston, MA 02110-1301
 *		USA
 */

#include "insytrack_cfg.h"

#define _BSD_SOURCE

#include <stddef.h>
#include <stdio.h>

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#ifdef HAVE_STRING_H
# if ((!defined STDC_HEADERS) || (!STDC_HEADERS)) && (defined HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>	/* strlen() */
#endif

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  ifdef HAVE_TIME_H
#   include <time.h>
#  endif
# endif
#endif

#include "insytrack/insytrack_client.h"

int randomize1 INSYTRACK_PARAMS ((int val));
int randomize2 INSYTRACK_PARAMS ((int val));
int randomize3 INSYTRACK_PARAMS ((int val));
int randomize4 INSYTRACK_PARAMS ((int val));
int start_random INSYTRACK_PARAMS ((void));

static insytrack_client_t insy_client;
static insytrack_client_error_t insy_error;
static char insy_id[] = "random_client";
static char insy_data[100];

/* ======================================================================== */

/**
 * Example processing function number 1.
 * @param val The value to process.
 * @return the processed value.
 */
int randomize1 (
#ifdef INSYTRACK_ANSIC
	int val)
#else
	val)
	int val;
#endif
{
#ifdef HAVE_SNPRINTF
	snprintf (insy_data, sizeof (insy_data) - 1, "randomize1: value=%d", val);
	insy_data[sizeof (insy_data) - 1] = '\0';
#else
# ifdef HAVE_SPRINTF
	sprintf (insy_data, "randomize1: value=%d", val);
	insy_data[sizeof (insy_data) - 1] = '\0';
# else
	insy_data[0] = '\0';
# endif
#endif
	insy_error = insytrack_mark_id (insy_client, insy_id,
		(unsigned short int)strlen (insy_id), insy_data,
		(unsigned short int)(strlen (insy_data) & 0x0FFFF));
	val &= 0x0FFFF;
	if ( 0x1000 < val && val < 0x8000 )
	{
		return randomize2 (val * 173 + 999);
	}
	if ( 0x8000 < val && val < 0xC000 )
	{
		return randomize3 (val * 5555 - 1111);
	}
	if ( 0xC000 < val )
	{
		return randomize4 (val * 8191 + 255);
	}
	return val;
}

/* ======================================================================== */

/**
 * Example processing function number 2.
 * @param val The value to process.
 * @return the processed value.
 */
int randomize2 (
#ifdef INSYTRACK_ANSIC
	int val)
#else
	val)
	int val;
#endif
{
#ifdef HAVE_SNPRINTF
	snprintf (insy_data, sizeof (insy_data) - 1, "randomize2: value=%d", val);
	insy_data[sizeof (insy_data) - 1] = '\0';
#else
# ifdef HAVE_SPRINTF
	sprintf (insy_data, "randomize2: value=%d", val);
	insy_data[sizeof (insy_data) - 1] = '\0';
# else
	insy_data[0] = '\0';
# endif
#endif
	insy_error = insytrack_mark_id (insy_client, insy_id,
		(unsigned short int)strlen (insy_id), insy_data,
		(unsigned short int)(strlen (insy_data) & 0x0FFFF));
	val &= 0x0FFFF;
	if ( 0x1000 < val && val < 0x8000 )
	{
		return randomize2 (val * 987 + 54321);
	}
	if ( 0x8000 < val && val < 0xC000 )
	{
		return randomize3 (val * 1234 - 5555);
	}
	if ( 0xC000 < val )
	{
		return randomize4 (val * 975 + 39775);
	}
	return val;
}

/* ======================================================================== */

/**
 * Example processing function number 3.
 * @param val The value to process.
 * @return the processed value.
 */
int randomize3 (
#ifdef INSYTRACK_ANSIC
	int val)
#else
	val)
	int val;
#endif
{
#ifdef HAVE_SNPRINTF
	snprintf (insy_data, sizeof (insy_data) - 1, "randomize3: value=%d", val);
	insy_data[sizeof (insy_data) - 1] = '\0';
#else
# ifdef HAVE_SPRINTF
	sprintf (insy_data, "randomize3: value=%d", val);
	insy_data[sizeof (insy_data) - 1] = '\0';
# else
	insy_data[0] = '\0';
# endif
#endif
	insy_error = insytrack_mark_id (insy_client, insy_id,
		(unsigned short int)strlen (insy_id), insy_data,
		(unsigned short int)(strlen (insy_data) & 0x0FFFF));
	val &= 0x0FFFF;
	if ( 0x1000 < val && val < 0x8000 )
	{
		return randomize2 (val * 4132 + 5623);
	}
	if ( 0x8000 < val && val < 0xC000 )
	{
		return randomize3 (val * 7823 - 135);
	}
	if ( 0xC000 < val )
	{
		return randomize4 (val * 12368 + 1325);
	}
	return val;
}

/* ======================================================================== */

/**
 * Example processing function number 4.
 * @param val The value to process.
 * @return the processed value.
 */
int randomize4 (
#ifdef INSYTRACK_ANSIC
	int val)
#else
	val)
	int val;
#endif
{
#ifdef HAVE_SNPRINTF
	snprintf (insy_data, sizeof (insy_data) - 1, "randomize4: value=%d", val);
	insy_data[sizeof (insy_data) - 1] = '\0';
#else
# ifdef HAVE_SPRINTF
	sprintf (insy_data, "randomize4: value=%d", val);
	insy_data[sizeof (insy_data) - 1] = '\0';
# else
	insy_data[0] = '\0';
# endif
#endif
	insy_error = insytrack_mark_id (insy_client, insy_id,
		(unsigned short int)strlen (insy_id), insy_data,
		(unsigned short int)(strlen (insy_data) & 0x0FFFF));
	val &= 0x0FFFF;
	if ( 0x1000 < val && val < 0x8000 )
	{
		return randomize2 (val * 42 + 127);
	}
	if ( 0x8000 < val && val < 0xC000 )
	{
		return randomize3 (val * 2364 - 4123);
	}
	if ( 0xC000 < val )
	{
		return randomize4 (val * 2348 + 12348);
	}
	return val;
}

/* ======================================================================== */

/**
 * A function that starts an example processing.
 * @return a generated value.
 */
int start_random (
#ifdef INSYTRACK_ANSIC
	void
#endif
)
{
	int value = 1;
	insy_error = insytrack_mark_id (insy_client, insy_id,
		(unsigned short int)strlen (insy_id), NULL, 0);

#if (defined HAVE_RANDOM) && (defined HAVE_SRANDOM) && (defined HAVE_TIME)
	srandom ((unsigned int)time (NULL));
	value = random () & 0x0FFFF;
#else
# if (defined HAVE_RAND) && (defined HAVE_SRAND) && (defined HAVE_TIME)
	srand ((unsigned int)time (NULL));
	value = rand () & 0x0FFFF;
# else
#  if (defined HAVE_TIME)
	value = (int)time (NULL) & 0x0FFFF;
#  endif
# endif
#endif
	return randomize1 (value);
}

/* ======================================================================== */

#ifndef INSYTRACK_ANSIC
int main INSYTRACK_PARAMS ((void));
#endif

/**
 * The entry point of the example InSyTrack client with pseudo-random flow.
 * @return an error code for the calling process.
 */
int main (
#ifdef INSYTRACK_ANSIC
	void
#endif
)
{
	insy_client = insytrack_client_create ();
	if ( insy_client == INSYTRACK_CLIENT_INVALID )
	{
		return 100;
	}

	insy_error = insytrack_client_start (insy_client);
	if ( insy_error != INSYTRACKC_OK )
	{
		return insy_error;
	}

	start_random ();
	insy_error = insytrack_mark_id_last (insy_client, insy_id,
		(unsigned short int)strlen (insy_id), NULL, 0);
	insytrack_client_close (insy_client);
	return insy_error;
}
