/*
 * InSyTrack - software for tracking program flow, server files.
 *	-- signal-related functions.
 *
 * Copyright (C) 2012-2013 Bogdan Drozdowski, bogdandr (at) op.pl
 * License: GNU Lesser General Public License, v3+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foudation:
 *		Free Software Foundation
 *		51 Franklin Street, Fifth Floor
 *		Boston, MA 02110-1301
 *		USA
 */

#include "insytrack_cfg.h"

#ifdef HAVE_STRING_H
# if ((!defined STDC_HEADERS) || (!STDC_HEADERS)) && (defined HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef HAVE_SIGNAL_H
# include <signal.h>
#endif

#include "insytrackd.h"
#include "insytrackd_signal.h"

/* ======================================================================== */

/* This has to be public and always defined: */
/**
 * A variable telling if a signal has been caught.
 */
volatile sig_atomic_t insytrackd_sig_recvd = 0;		/* non-zero after signal received */

#ifdef HAVE_SIGNAL_H
# if (defined HAVE_SIGACTION) && (!defined __STRICT_ANSI__)
static struct sigaction sa/* = { .sa_handler = &term_signal_received }*/;
# endif
/* Handled signals which will cause the program to exit cleanly. */
static const int signals[] =
{
	SIGINT
# ifdef SIGQUIT
	, SIGQUIT
# endif
# ifdef SIGILL
	, SIGILL
# endif
# ifdef SIGABRT
	, SIGABRT
# endif
# ifdef SIGFPE
	, SIGFPE
# endif
# ifdef SIGSEGV
	, SIGSEGV
# endif
# ifdef SIGPIPE
	, SIGPIPE
# endif
# ifdef SIGALRM
	, SIGALRM
# endif
# ifdef SIGTERM
	, SIGTERM
# endif
# ifdef SIGUSR1
	, SIGUSR1
# endif
# ifdef SIGUSR2
	, SIGUSR2
# endif
# ifdef SIGTTIN
	, SIGTTIN
# endif
# ifdef SIGTTOU
	, SIGTTOU
# endif
# ifdef SIGBUS
	, SIGBUS
# endif
# ifdef SIGPROF
	, SIGPROF
# endif
# ifdef SIGSYS
	, SIGSYS
# endif
# ifdef SIGTRAP
	, SIGTRAP
# endif
# ifdef SIGXCPU
	, SIGXCPU
# endif
# ifdef SIGXFSZ
	, SIGXFSZ
# endif
# ifdef SIGVTALRM
	, SIGVTALRM
# endif
# ifdef SIGCHLD
	, SIGCHLD
# endif
# ifdef SIGPOLL
	, SIGPOLL
# endif
# ifdef SIGPWR
	, SIGPWR
# endif
# ifdef SIGUNUSED
	, SIGUNUSED
# endif
# ifdef SIGEMT
	, SIGEMT
# endif
# ifdef SIGLOST
	, SIGLOST
# endif
# ifdef SIGIO
	, SIGIO
# endif
};

# ifndef RETSIGTYPE
#  define RETSIGTYPE void
# endif

/* =============================================================== */

# ifndef INSYTRACK_ANSIC
static RETSIGTYPE term_signal_received INSYTRACK_PARAMS ((const int signum));
# endif

/**
 * Signal handler - Sets a flag which will stop further program operations,
 * when a signal which would normally terminate the program is received.
 * @param signum Signal number.
 */
static RETSIGTYPE
term_signal_received (
#ifdef INSYTRACK_ANSIC
	const int signum)
# else
	signum)
	const int signum;
# endif
{
	insytrackd_sig_recvd = signum;
# define void 1
# define int 2
# if RETSIGTYPE != void
	return 0;
# endif
# undef int
# undef void
}

#endif /* HAVE_SIGNAL_H */

/* =============================================================== */

/**
 * Sets the signal handler for all available signals.
 */
void insytrackd_set_sigh (
#ifdef INSYTRACK_ANSIC
	void
#endif
)
{
#ifdef HAVE_SIGNAL_H
# define 	TMPSIZE	12
	size_t s;			/* sizeof(signals) */
# if (!defined HAVE_SIGACTION) || (defined __STRICT_ANSI__)
	typedef void (*sighandler_t) (int);
	sighandler_t shndlr;
# endif
# ifndef HAVE_MEMSET
	size_t i;
# endif
	/*
	 * Setting signal handlers.
	 */

# if (!defined HAVE_SIGACTION) || (defined __STRICT_ANSI__)
	/* ANSI C */
	for ( s=0; s < sizeof (signals) / sizeof (signals[0]); s++ )
	{
		shndlr = signal ( signals[s], &term_signal_received );
	}
# else
	/* more than ANSI C */
#  ifdef HAVE_MEMSET
	memset (&sa, 0, sizeof (struct sigaction));
#  else
	for ( i=0; i < sizeof (struct sigaction); i++ )
	{
		((char *)&sa)[i] = '\0';
	}
#  endif
	sa.sa_handler = &term_signal_received;
	for ( s=0; s < sizeof (signals) / sizeof (signals[0]); s++ )
	{
		sigaction ( signals[s], &sa, NULL);
	}
# endif		/* ! ANSI C */

#endif /* HAVE_SIGNAL_H */
}
