/*
 * InSyTrack - software for tracking program flow, client files.
 *	-- client initialization & shutdown, header file.
 *
 * Copyright (C) 2012-2013 Bogdan Drozdowski, bogdandr (at) op.pl
 * License: GNU Lesser General Public License, v3+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foudation:
 *		Free Software Foundation
 *		51 Franklin Street, Fifth Floor
 *		Boston, MA 02110-1301
 *		USA
 */

#ifndef _LIBINSYTRACK_CLIENT_INIT_H
# define _LIBINSYTRACK_CLIENT_INIT_H

#include "insytrack_cfg.h"

#ifdef HAVE_PTHREAD_H
# include <pthread.h>
#endif

# include "insytrack_client.h"

/**
 * The enumeration of possible client states.
 */
enum insytrack_client_state
{
	INSYTRACKC_INIT,
	INSYTRACKC_STARTED
};

/**
 * The type of possible client states.
 */
typedef enum insytrack_client_state insytrack_client_state_t;

/**
 * This is what insytrack_client_t is cast from/to.
 */
struct insytrack_client
{
	/**
	 * The client's socket's domain.
	 */
	int insy_socket_domain;

	/**
	 * The client's socket's type.
	 */
	int insy_socket_type;

	/**
	 * The client's socket's protocol.
	 */
	int insy_socket_protocol;

	/**
	 * The server's socket's connection address.
	 */
	struct sockaddr * insy_socket_connaddr;

	/**
	 * The server's socket's connection address' length.
	 */
	socklen_t insy_connaddr_len;

	/**
	 * A flag telling whether insy_socket_connaddr was allocated by us.
	 */
	int insy_connaddr_alloc;

	/**
	 * The client's socket.
	 */
	int insy_client_socket;

	/**
	 * The current state of the client.
	 */
	insytrack_client_state_t insy_state;

	/**
	 * The mutex for performing operations on the client.
	 */
	pthread_mutex_t insy_client_mutex;
};

#endif /* _LIBINSYTRACK_CLIENT_INIT_H */
