/*
 * InSyTrack - software for tracking program flow, server files.
 *	-- server initialization & shutdown, header file.
 *
 * Copyright (C) 2012-2013 Bogdan Drozdowski, bogdandr (at) op.pl
 * License: GNU Lesser General Public License, v3+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foudation:
 *		Free Software Foundation
 *		51 Franklin Street, Fifth Floor
 *		Boston, MA 02110-1301
 *		USA
 */

#ifndef _LIBINSYTRACK_SERVER_INIT_H
# define _LIBINSYTRACK_SERVER_INIT_H

#include "insytrack_cfg.h"

#ifdef HAVE_PTHREAD_H
# include <pthread.h>
#endif

# include "insytrack_server.h"

/**
 * The number of seconds per each select() call.
 */
# define INSYTRACK_MAX_SELECT_SECONDS 1


# if (((defined HAVE_SYS_SELECT_H) || (((defined TIME_WITH_SYS_TIME)	\
	|| (defined HAVE_SYS_TIME_H) || (defined HAVE_TIME_H))		\
	&& (defined HAVE_UNISTD_H)))				\
	&& (defined HAVE_SELECT))
#  define INSYTRACK_HAVE_SELECT 1
# else
#  undef INSYTRACK_HAVE_SELECT
# endif

/**
 * The enumeration of possible server states.
 */
enum insytrack_server_state
{
	INSYTRACKS_INIT,
	INSYTRACKS_STARTED
};

/**
 * The type of possible server states.
 */
typedef enum insytrack_server_state insytrack_server_state_t;

/**
 * This is what insytrack_server_t is cast from/to.
 */
struct insytrack_server
{
	/**
	 * The server's socket's domain.
	 */
	int insy_socket_domain;

	/**
	 * The server's socket's type.
	 */
	int insy_socket_type;

	/**
	 * The server's socket's protocol.
	 */
	int insy_socket_protocol;

	/**
	 * The server's socket's binding address.
	 */
	struct sockaddr * insy_socket_bindaddr;

	/**
	 * The server's socket's binding address' length.
	 */
	socklen_t insy_bindaddr_len;

	/**
	 * A flag telling whether insy_socket_bindaddr was allocated by us.
	 */
	int insy_bindaddr_alloc;

	/**
	 * The server's socket's backlog.
	 */
	int insy_backlog;

	/**
	 * The server's connection dispatcher thread.
	 */
	pthread_t insy_dispatch_thread;

	/**
	 * The server's socket.
	 */
	int insy_server_socket;

	/**
	 * A flag telling if the server's shutting down.
	 */
	volatile int insy_shutting_down;

	/**
	 * A list of the server's connection threads.
	 */
	struct insytrack_thread_list * insy_tlist_head;

	/**
	 * The mutex for the list of the server's connection threads.
	 */
	pthread_mutex_t insy_list_mutex;

	/**
	 * The "message received" callback function.
	 */
	insytrack_msg_recvd_fun_t insy_msgrecvd_callback;

	/**
	 * The user-provided data passed to the "message received"
	 * callback function.
	 */
	void * insy_msgrecvd_callback_data;

	/**
	 * The "client connected" callback function.
	 */
	insytrack_client_connected_fun_t insy_conn_callback;

	/**
	 * The user-provided data passed to the "client connected"
	 * callback function.
	 */
	void * insy_conn_callback_data;

	/**
	 * The "client disconnected" callback function.
	 */
	insytrack_client_disconnected_fun_t insy_disconn_callback;

	/**
	 * The user-provided data passed to the "client disconnected"
	 * callback function.
	 */
	void * insy_disconn_callback_data;

	/**
	 * The current state of the server.
	 */
	insytrack_server_state_t insy_state;

	/**
	 * The mutex for performing operations on the server.
	 */
	pthread_mutex_t insy_server_mutex;
};

#endif /* _LIBINSYTRACK_SERVER_INIT_H */
