/*
 * InSyTrack - software for tracking program flow, server files.
 *	-- server worker threads' header file.
 *
 * Copyright (C) 2012-2013 Bogdan Drozdowski, bogdandr (at) op.pl
 * License: GNU Lesser General Public License, v3+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foudation:
 *		Free Software Foundation
 *		51 Franklin Street, Fifth Floor
 *		Boston, MA 02110-1301
 *		USA
 */

#ifndef _LIBINSYTRACK_SERVER_THREAD_H
# define _LIBINSYTRACK_SERVER_THREAD_H 1

# include "insytrack_server.h"
# include "libinsytrack_server_init.h"

# ifdef __cplusplus
extern "C" {
# endif

/**
 * The data passed to a worker thread.
 */
struct insytrack_worker_data
{
	/**
	 * The server of the thread.
	 */
	struct insytrack_server * insy_server;

	/**
	 * The client socket descriptor to use for communication.
	 */
	int insy_client_socket;

	/**
	 * The client socket address.
	 */
	struct sockaddr * insy_client_addr;

	/**
	 * The client socket address length.
	 */
	socklen_t insy_client_addr_len;

	/**
	 * A flag telling if the thread is running.
	 */
	volatile int insy_is_running;

	/**
	 * The working buffer.
	 */
	unsigned char * insy_buf;

	/**
	 * The working buffer's total size.
	 */
	size_t insy_bufsize;

	/**
	 * An index into insy_buf telling where the current start is.
	 */
	size_t insy_bufhead;

	/**
	 * The length of received data in insy_buf.
	 */
	size_t insy_currlen;
};

/**
 * The connection worker thread.
 * @param insy_param The parameter passed by pthread_create().
 * @return NULL.
 */
extern void *
insytrack_worker INSYTRACK_PARAMS ((void * insy_param));

# ifdef __cplusplus
}
# endif

#endif /* _LIBINSYTRACK_SERVER_THREAD_H */
