/*
 * InSyTrack - software for tracking program flow, server files.
 *	-- server threads' list, header file.
 *
 * Copyright (C) 2012-2013 Bogdan Drozdowski, bogdandr (at) op.pl
 * License: GNU Lesser General Public License, v3+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foudation:
 *		Free Software Foundation
 *		51 Franklin Street, Fifth Floor
 *		Boston, MA 02110-1301
 *		USA
 */

#ifndef _LIBINSYTRACK_SERVER_TLIST_H
# define _LIBINSYTRACK_SERVER_TLIST_H 1

# include "insytrack_cfg.h"
# include "insytrack_server.h"
# include "libinsytrack_server_thread.h"

# ifdef HAVE_PTHREAD_H
#  include <pthread.h>
# endif

/**
 * An element of the thread list.
 */
struct insytrack_thread_list
{
	/**
	 * The thread's ID.
	 */
	pthread_t insy_thread_id;

	/**
	 * The data passed to the thread.
	 */
	struct insytrack_worker_data * insy_thread_data;

	/**
	 * Next element of the list.
	 */
	struct insytrack_thread_list * insy_next;

	/**
	 * Previous element of the list.
	 */
	struct insytrack_thread_list * insy_prev;
};

# include "libinsytrack_server_init.h"

# ifdef __cplusplus
extern "C" {
# endif

/**
 * Creates a new worker thread and adds it to the server's list.
 * @param insy_server The server to add the thread to.
 * @param insy_twd The thread's working data.
 * @return an insytrack_server_error_t value specifying the
 *	status of the operation.
 */
extern GCC_WARN_UNUSED_RESULT insytrack_server_error_t
insytrack_create_worker	INSYTRACK_PARAMS
	((struct insytrack_server * insy_server,
	struct insytrack_worker_data * insy_twd));

/**
 * Removes a worker thread from the server's list.
 * @param insy_server The server to remove the thread from.
 * @param insy_thread The thread to remove.
 * @return an insytrack_server_error_t value specifying the
 *	status of the operation.
 */
extern insytrack_server_error_t
insytrack_remove_worker	INSYTRACK_PARAMS
	((struct insytrack_server * insy_server,
	pthread_t insy_thread));

# ifdef __cplusplus
}
# endif

#endif /* _LIBINSYTRACK_SERVER_TLIST_H */
